# SPDX-License-Identifier: Apache-2.0
"""Bundled search profiles for Zyra discovery backends.

Profiles define sources (local catalogs, OGC WMS/Records endpoints) and
optional scoring weights used by the discovery backends.
"""
